/**
 * @File
 *
 * Javascript handler for livetracking map.
 *
 * Prepare the map, receive a message from Nodejs and add the position.
 * Toggle connected / disconnected.
 * Show last position.
 *
 * @todo Display track polyline as well.
 */

(function($) {
  var ol;
  var pointFeature;
  var vectorLayer;
  var autoZoom = true;
  var lastTime;

  Drupal.behaviors.nodejs_livetracking = {
    'attach': function(context, setting) {
      if (context == document) {
        ol = $('#openlayers-map').data('openlayers');
        var mapStyle = new OpenLayers.StyleMap(ol.map.styles);
        vectorLayer = new OpenLayers.Layer.Vector('User position', {styleMap: mapStyle});
        ol.openlayers.addLayer(vectorLayer);
        $('#followme').bind('change', function() {
          autoZoom = !autoZoom;
        });

        setInterval(function() {
          $('#timestamp').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime))
        }, 5000);
      }
     },

    // Get latest marker.
    'getPoint' : function(){
      return pointFeature;
    },

    // Set latest marker.
    'setPoint' : function(feature){
      pointFeature = feature;
    },

    // Zoom the feature (Follow).
    'zoomToFeature' : function(){
      if (autoZoom == true){
        ol.openlayers.zoomToExtent(vectorLayer.getDataExtent());
      }
    },

    'setPresence' : function(bool){
      if (bool == true) {
        $("#connected").removeClass('hidden');
        $("#disconnected").addClass('hidden');
      }
      else {
        $("#connected").addClass('hidden');
        $("#disconnected").removeClass('hidden');
      }
    },

    'setLastTime' : function(time){
      lastTime = time;
       $('#timestamp').html(Drupal.behaviors.nodejs_livetracking.timeAgo(lastTime));
    },

    // Return time ago from timestamp. Copied and adapted from : https://gist.github.com/skattyadz/1285806
    'timeAgo' : function(time){
      if (time == undefined) {
        return Drupal.t("Unknown");
      }
      var units = [
        { name: "second", limit: 60, in_seconds: 1 },
        { name: "minute", limit: 3600, in_seconds: 60 },
        { name: "hour", limit: 86400, in_seconds: 3600 },
        { name: "day", limit: 604800, in_seconds: 86400 },
        { name: "week", limit: 2629743, in_seconds: 604800 },
        { name: "month", limit: 31556926, in_seconds: 2629743 },
        { name: "year", limit: null, in_seconds: 31556926 }
      ];

      var diff = (new Date() - new Date(time)) / 1000;
      if (diff < 5) {
        return Drupal.t("just now");
      }
      var i = 0;
      while (unit = units[i++]) {
        if (diff < unit.limit || !unit.limit){
          var diff = Math.floor(diff / unit.in_seconds);
          return diff + " " + Drupal.t(unit.name) + (diff > 1 ? "s" : "") + " " + Drupal.t("old");
        }
      };
    },
  };

  Drupal.Nodejs.callbacks.nodejsGeoloc = {
    callback: function (message) {
      if (message.messageType == 'device-auth') {
        Drupal.behaviors.nodejs_livetracking.setPresence(true);
        message['data'] = {
          'body' : Drupal.t('User @id is now here !', {'@id' : message.uid}),
          'subject' : Drupal.t('Device connected')
        };
        Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
      }
      else if (message.messageType == 'device-off') {
        Drupal.behaviors.nodejs_livetracking.setPresence(false);
        message['data'] = {
          'body' : Drupal.t('We just lost user @id !', {'@id' : message.uid}),
          'subject' : Drupal.t('Device disconnected')
        };
        Drupal.Nodejs.callbacks.nodejsNotify.callback(message);
        // There is no position in this message. I just quit now.
        return;
      }
      var point = new OpenLayers.Geometry.Point(message.position.lon, message.position.lat).transform("EPSG:4326", ol.map.projection);
      var pointFeature = Drupal.behaviors.nodejs_livetracking.getPoint();
      if (pointFeature != undefined) {
        vectorLayer.removeFeatures([pointFeature]);
      }
      pointFeature = new OpenLayers.Feature.Vector(point);
      Drupal.behaviors.nodejs_livetracking.setPoint(pointFeature);
      vectorLayer.addFeatures([pointFeature]);
      vectorLayer.refresh();
      Drupal.behaviors.nodejs_livetracking.zoomToFeature();
      for (var key in message.position) {
        if (key != 'timestamp' || key != 'uuid') {
          $('#' + key).html(message.position[key]);
        }
      }
      Drupal.behaviors.nodejs_livetracking.setLastTime(message.position.timestamp);
    },
  };
})(jQuery);
