/**
 * @File
 * Livetracking server extension for nodejs
 *
 * IMPORTANT ! Add this extension name into node.config.js
 */

/**
 * There is many node caching module around. I started with node-cache tryed memory-cache but finally decided to switch to expirable instead.
**/

var Expirable = require('expirable');
var myCache = new Expirable('3 minutes');

// config get from config export.setup
var configuration;

/**
 * Helper function that check for errors in backend response.
 */
var checkErrors = function (error, response, body, type) {
  var authData = false;
  if (error) {
    if (configuration.debug == true) {
      console.log("Error with " + type + " on device request:", error);
    }
    return authData;
  }
  if (response.statusCode == 404) {
    if (configuration.debug == true){
      console.log('Backend ' + type + ' url not found, full response info:', response);
    }
    return authData;
  }
  try {
    authData = JSON.parse(body);
  }
  catch(exception) {
    if (configuration.debug == true) {
      console.log('Failed to parse ' + type + ' message:', exception);
      console.log('Failed message string: ' + body);
    }
    return authData;
  }

  return authData;
}

/**
 * Handle authentication device call response.
 */
var authDeviceCallback = function (error, response, body) {

  var authData = checkErrors(error, response, body, 'authentication');
  if (authData == false) {
    return;
  }

  var key = authData.shift();
  myCache.set(key, authData);
  // The expired boolean tells if the key was removed because it was expired or if it was a manual removal
  myCache.on(key + ':removed', function(expired) {
    var offMes = {
      messageType: 'device-off',
      uuid : key,
    };
    // => Set device Offline.
    configuration.sendMessageToBackend(offMes);
  });
}

/**
 * Handle incoming message entering from route "/"
 */
var trackAuth = function(request, reponse) {
  // Parsing the response to get JSON.
  var requestBody = '';
  request.setEncoding('utf8');
  request.on('data', function (chunk) {
    requestBody += chunk;
  });
  request.on('end', function () {
    try {
      // Got Json.
      var message = JSON.parse(requestBody);
      // From here I will have to process the message to get a unique and similar message object for all supported tracking device.
      // message = processPosition(message);
    }
    catch(exception) {
      if (configuration.debug == true) {
        console.log('trackAuth: Invalid JSON "' + requestBody + '"',  exception);
      }
      return;
    }

    // In case uuid property exist. (message coming from FFVL PhoneGap app !).
    if (message.hasOwnProperty('uuid')) {
      var uuidIsCached = myCache.has(message.uuid) ? true : false;

      // In case this device has not been cached.
      if (uuidIsCached == false) {
        // Cache it immediatly to avoid sending multiple authentication message.
        myCache.set(message.uuid, "Pending Authentication");
        var authMes = {
          messageType: 'device-auth',
          uuid : message.uuid,
        };
        // => Authenticate the device.
        configuration.sendMessageToBackend(authMes, authDeviceCallback);
      }
      else {
        // This automatically give x minutes more ttl to this key ! cf. Expirable nodejs module.
        var value = myCache.get(message.uuid);
        // If device is still pending authentication. Ignore this message.
        if (value == "Pending Authentication") {
          return;
        }
        posMes = {
          messageType: 'device-pos',
          position : message,
          channel : value[1],
          uid : value[0],
        }
        // => Send the position.
        sendMessageToTokenChannel(posMes);
      }
    }
  });
}

/**
 * Send message to contentChannel via Nodejs.
 */
function sendMessageToTokenChannel(message) {
  if (!message.hasOwnProperty('channel')) {
    console.log('publishMessageToContentChannel: An invalid message object was provided.');
    return;
  }
  if (!configuration.tokenChannels.hasOwnProperty(message.channel)) {
    console.log('publishMessageToContentChannel: The channel "' + message.channel + '" doesn\'t exist.');
    return;
  }
  for (var socketId in configuration.tokenChannels[message.channel].sockets) {
    configuration.publishMessageToClient(socketId, message);
  }
}

// Here I'll process the message depending on whitch trackeur type it is.
var processPosition = function(mess) {
  // If it's from PhoneGap... Don't touch.
  // else process mess to get a PhoneGap like message !
  // return message;
}

// Define a route that will receive tracking message.
exports.routes = Array({
  path : '/',
  type : 'post',
  handler : trackAuth,
});

exports.setup = function(config) {
  configuration = config;
}
